#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "basic_io.h"

File * open_file(char * file_name, int flags, int mode){

	File * file = (File *) malloc(sizeof(File));
	
	file->fd = open(file_name, flags, mode);

	if(file->fd < 0) {

		fprintf(stderr, "Error opening file '%s'\n", file_name);
		return NULL;
	}

	return file;
}

File * open_for_read(char * fileName){

	return open_file(fileName, O_RDWR | O_DIRECT, 0);
}

File * open_for_write(char * fileName){

	return open_file(fileName, O_WRONLY | O_DIRECT | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
}

void seek(File * file, int offset){

	lseek(file->fd, offset, SEEK_SET);
}

void close_file(File * file){

	close(file->fd);
}

#define SECTOR_SIZE 512

unsigned char * alloc_alligned_buffer(int size){

	unsigned char * buffer = NULL;
	long raw_buffer_address = (long) malloc (size + SECTOR_SIZE);

	if(raw_buffer_address % SECTOR_SIZE == 0){

		buffer = (unsigned char *) raw_buffer_address;
	}
	else{

		buffer = (unsigned char *) (((raw_buffer_address + SECTOR_SIZE) / SECTOR_SIZE ) * SECTOR_SIZE);
	}

	return buffer;
}

int read_bytes(File * file, unsigned char * buffer, int n_bytes){

	return read(file->fd, buffer, n_bytes);
}

int write_bytes(File * file, unsigned char * buffer, int n_bytes){

	return write(file->fd, buffer, n_bytes);
}
